# Arctic Stripes
# Copyright (C) 2021  Damien Ringeisen

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

import numpy as np
import pylab as plt
from netCDF4 import Dataset
from copy import deepcopy
import scipy as sp
import scipy.ndimage

from mpl_toolkits.basemap import Basemap

# Using LaTeX in figures
plt.rc('text', usetex=True)
font = {'family' : 'serif',
        'weight' : 'bold',
        'size'   : 16}
plt.rc('font', **font)


# data file
data_f = './zonmap_monthly2.nc'

# loading the data
data = Dataset(data_f, mode='r')

# print data info
print(data)

# defining the coordinate for plotting years on basemap
lon=(np.array(data['time'])-1960.)/(2020-1960)*360.-180

# Latitude
lat=np.array(data['lat'])

# Paralleles for the zones
zon_lats=[-90,-64,-24,24,64,90]


############### Temperature data preparation for the plots

# Original Temperature data
temp=data['temperature']

# Deepcopy of the temperature data
temp2=deepcopy(np.array(data['temperature']))

# Interpolation in the y axis
temp2c=(temp2[1:,:]+temp2[:-1,:])/2.
tempc=(temp[1:,:]+temp[:-1,:])/2.
temp3=deepcopy(tempc)

# sizes of the temperature data
s=temp2.shape

### Temperature average in the zones

# Definition of the zones
pol2 =[0,6]
mid2 =[6,16]
trop=[16,28]
mid1 =[28,38]
pol1 =[38,45]

# latitude of the average
latc=(lat[1:] + lat[:-1]) / 2.

# Factor for surface correction
A_fac=(2*np.pi*np.sin((np.abs(latc)+90)/180.*np.pi)*(4./180*np.pi))/(4.*np.pi)

# sum of correction factor for the zone (for the weight)
A_trop = np.sum(A_fac[trop[0]:trop[1]])
A_pol1 = np.sum(A_fac[pol1[0]:pol1[1]])
A_pol2 = np.sum(A_fac[pol2[0]:pol2[1]])
A_mid1 = np.sum(A_fac[mid1[0]:mid1[1]])
A_mid2 = np.sum(A_fac[mid2[0]:mid2[1]])

for t in range(s[1]):
    # average weighted with correction factor
    temp2c[trop[0]:trop[1],t]=np.sum(temp2c[trop[0]:trop[1],t]*A_fac[trop[0]:trop[1]])/A_trop
    temp2c[pol1[0]:pol1[1],t]=np.sum(temp2c[pol1[0]:pol1[1],t]*A_fac[pol1[0]:pol1[1]])/A_pol1
    temp2c[pol2[0]:pol2[1],t]=np.sum(temp2c[pol2[0]:pol2[1],t]*A_fac[pol2[0]:pol2[1]])/A_pol2
    temp2c[mid1[0]:mid1[1],t]=np.sum(temp2c[mid1[0]:mid1[1],t]*A_fac[mid1[0]:mid1[1]])/A_mid1
    temp2c[mid2[0]:mid2[1],t]=np.sum(temp2c[mid2[0]:mid2[1],t]*A_fac[mid2[0]:mid2[1]])/A_mid2

# Copy of data for yearly averaging
temp2cs=deepcopy(temp2c)

# yearly averaging
for t in range(60):
    temp2cs[trop[0]:trop[1],12*t:12*(t+1)]=np.mean(temp2c[trop[0]:trop[1],12*t:12*(t+1)])
    temp2cs[pol1[0]:pol1[1],12*t:12*(t+1)]=np.mean(temp2c[pol1[0]:pol1[1],12*t:12*(t+1)])
    temp2cs[pol2[0]:pol2[1],12*t:12*(t+1)]=np.mean(temp2c[pol2[0]:pol2[1],12*t:12*(t+1)])
    temp2cs[mid1[0]:mid1[1],12*t:12*(t+1)]=np.mean(temp2c[mid1[0]:mid1[1],12*t:12*(t+1)])
    temp2cs[mid2[0]:mid2[1],12*t:12*(t+1)]=np.mean(temp2c[mid2[0]:mid2[1],12*t:12*(t+1)])


temp2c[:,-11:-1]=np.mean(temp2c[:,-11:-1])

### Global Temperature average

s=temp3.shape

for t in range(s[1]):
    temp3[:,t]=np.sum(temp3[:,t]*A_fac)

temp3s=deepcopy(temp3)

for t in range(60):
    temp3s[:,12*t:12*(t+1)]=np.mean(temp3[:,12*t:12*(t+1)])


############### Original data plot with colorbar and years

fig, ax = plt.subplots(num=None, figsize=(10, 6) )
m = Basemap(projection='cyl',llcrnrlat=-90,urcrnrlat=90,llcrnrlon=-180,urcrnrlon=179.4911186440679,resolution='c')

v_cbar_ori=max(np.nanmax(temp),abs(np.nanmax(temp)))
print('v_cbar_ori = ',v_cbar_ori)
# v_cbar=0.7
fac=1.

xo, yo = m(*np.meshgrid(lon,lat))
plot=m.pcolormesh(xo,yo,temp,cmap='RdBu_r',vmin=-fac*v_cbar_ori,vmax=fac*v_cbar_ori,shading='flat')
cbar=m.colorbar(plot)
cbar.set_label(r'$^\circ$C')

labels=np.arange(1960,2021,10)

ticks=(labels-1960.)/(2020-1960)*360.-180

# m.drawmeridians(ticks,labels=[0,0,0,0])
m.drawparallels([-90,-64,-24,24,64,90],labels=[1,0,0,0])


plt.xticks(ticks=ticks,labels=labels)

plt.tight_layout()

plt.title('1-month zonal mean anomalies vs 1960-2010')

fig.savefig('figs/original_data.png', dpi=300)

############### GLOBAL EARTH YEARLY plot without colorbar and years

fig, ax = plt.subplots(num=None, figsize=(10, 6) )
m = Basemap(projection='cyl',llcrnrlat=-90,urcrnrlat=90,llcrnrlon=-180,urcrnrlon=179.4911186440679,resolution='c')

v_cbar_glob=max(np.nanmax(temp3s),abs(np.nanmax(temp3s)))
print('v_cbar_glob = ',v_cbar_glob)
# v_cbar=0.7
fac=1.

x, y = m(*np.meshgrid(lon,latc))
plot=m.pcolormesh(x,y,temp3s,cmap='RdBu_r',vmin=-fac*v_cbar_glob,vmax=fac*v_cbar_glob,shading='flat')

plt.tight_layout()

ax.axis('off')

fig.savefig('figs/annual_stripes_global.png', dpi=300)

############### GLOBAL EARTH YEARLY plot with colorbar and years

fig, ax = plt.subplots(num=None, figsize=(10, 6) )
m = Basemap(projection='cyl',llcrnrlat=-90,urcrnrlat=90,llcrnrlon=-180,urcrnrlon=179.4911186440679,resolution='c')

# v_cbar=max(np.nanmax(temp3s),abs(np.nanmax(temp3s)))
# v_cbar=0.7
fac=1.

x, y = m(*np.meshgrid(lon,latc))
plot=m.pcolormesh(x,y,temp3s,cmap='RdBu_r',vmin=-fac*v_cbar_glob,vmax=fac*v_cbar_glob,shading='flat')
cbar=m.colorbar(plot)
cbar.set_label(r'$^\circ$C')

labels=np.arange(1960,2021,10)

ticks=(labels-1960.)/(2020-1960)*360.-180

# m.drawmeridians(ticks,labels=[0,0,0,0])

plt.xticks(ticks=ticks,labels=labels)

plt.tight_layout()

fig.savefig('figs/annual_stripes_global_years+colorbar.png', dpi=300)

############### ZONES EARTH YEARLY plot with colorbar and years

fig, ax = plt.subplots(nrows=1, ncols=1,  num=None, figsize=(10, 6) )

m = Basemap(projection='cyl',llcrnrlat=-90,urcrnrlat=90,llcrnrlon=-180,urcrnrlon=178.4911186440679,resolution='c',ax=ax)

v_cbar_zones=max(np.nanmax(temp2cs),abs(np.nanmin(temp2cs)))
# v_cbar=0.7
fac=1.

x, y = m(*np.meshgrid(lon,latc))
plot=m.pcolormesh(x,y,temp2cs, cmap='RdBu_r', vmin=-fac*v_cbar_zones, vmax=fac*v_cbar_zones, shading='flat')
cbar=m.colorbar(plot)
cbar.set_label(r'$^\circ$C')
m.drawmapboundary()

m.drawparallels([-90,-64,-24,24,64,90],labels=[1,0,0,0])

labels=np.arange(1960,2021,10)

ticks=(labels-1960.)/(2020-1960)*360.-180

# m.drawmeridians(ticks,labels=[0,0,0,0])

plt.xticks(ticks=ticks,labels=labels)

plt.xlabel('Years')

plt.tight_layout()

fig.savefig('figs/stripes_zones_fullbar.png', dpi=300)

############### ZONES EARTH YEARLY plot with global legends

fig, ax = plt.subplots(nrows=1, ncols=1,  num=None, figsize=(10, 6) )

m = Basemap(projection='cyl',llcrnrlat=-90,urcrnrlat=90,llcrnrlon=-180,urcrnrlon=178.4911186440679,resolution='c',ax=ax)

fac=1.

x, y = m(*np.meshgrid(lon,latc))
plot=m.pcolormesh(x,y,temp2cs,cmap='RdBu_r',vmin=-fac*v_cbar_glob,vmax=fac*v_cbar_glob, shading='flat')
cbar=m.colorbar(plot)
cbar.set_label(r'$^\circ$C')
m.drawmapboundary()

m.drawparallels([-90,-64,-24,24,64,90],labels=[1,0,0,0])

labels=np.arange(1960,2021,10)

ticks=(labels-1960.)/(2020-1960)*360.-180

# m.drawmeridians(ticks,labels=[0,0,0,0])

plt.xticks(ticks=ticks,labels=labels)

plt.xlabel('Years')

plt.tight_layout()

fig.savefig('figs/stripes_zones_globbar.png', dpi=300)

############### ZONES EARTH YEARLY plot end poster

fig, ax = plt.subplots(nrows=2, ncols=1,  num=None, figsize=(10, 8), gridspec_kw={'height_ratios': [5, 1]})

m = Basemap(projection='cyl',llcrnrlat=-90,urcrnrlat=90,llcrnrlon=-180,urcrnrlon=178.4911186440679,resolution='c',ax=ax[0])

v_cbar_zones=max(np.nanmax(temp2cs),abs(np.nanmin(temp2cs)))
print('v_cbar_zones = ',v_cbar_zones)
fac=1.

x, y = m(*np.meshgrid(lon,latc))
plot=m.pcolormesh(x,y,temp2cs,cmap='RdBu_r',vmin=-fac*v_cbar_zones,vmax=fac*v_cbar_zones, shading='flat')

plt.rcParams.update({
    "text.usetex": True,
    "font.family": "sans-serif",
    "font.sans-serif": ["Computer Modern Sans Serif"]})

font = {'family': 'sans-serif',
        'color':  'black',
        # 'weight': 'bold',
        'size': 20
        }

box = {'facecolor': 'white',
       'edgecolor': 'None',
       'boxstyle': 'square'
      }

ax[0].text(0, 76, r'\textbf{Arctic}', fontdict=font, bbox=box, horizontalalignment='center',verticalalignment='center')
ax[0].text(0, 45, r'\textbf{Northern Midlatitudes}', fontdict=font, bbox=box, horizontalalignment='center',verticalalignment='center')
ax[0].text(0, 0, r'\textbf{Tropics}', fontdict=font, bbox=box, horizontalalignment='center',verticalalignment='center')
ax[0].text(0, -45, r'\textbf{Southern Midlatitudes}', fontdict=font, bbox=box, horizontalalignment='center',verticalalignment='center')
ax[0].text(0, -76, r'\textbf{Antarctic}', fontdict=font, bbox=box, horizontalalignment='center',verticalalignment='center')

m = Basemap(projection='cyl',llcrnrlat=-20,urcrnrlat=20,llcrnrlon=-180,urcrnrlon=178.4911186440679,resolution='c',ax=ax[1])
x, y = m(*np.meshgrid(lon,latc))

ax[1].pcolormesh(x,y,temp3s,cmap='RdBu_r',vmin=-fac*v_cbar_zones,vmax=fac*v_cbar_zones, shading='flat')
ax[1].text(0, 0, r'\textbf{Global}', fontdict=font, bbox=box, horizontalalignment='center',verticalalignment='center')

ax[0].axis('off')
ax[1].axis('off')

fig.tight_layout()

fig.savefig('figs/stripes_zones_final.png', dpi=300)

############### Empty map

fig, ax = plt.subplots(num=None, figsize=(10, 6) )
m = Basemap(projection='cyl',llcrnrlat=-90,urcrnrlat=90,llcrnrlon=-180,urcrnrlon=179.4911186440679,resolution='c')

m.shadedrelief(scale=0.2)

fig.savefig('figs/earth_map-noz.png', dpi=300)

############### Map with parralels defining the zones

fig, ax = plt.subplots(num=None, figsize=(10, 6) )
m = Basemap(projection='cyl',llcrnrlat=-90,urcrnrlat=90,llcrnrlon=-180,urcrnrlon=179.4911186440679,resolution='c')

m.shadedrelief(scale=0.2)

m.drawparallels(zon_lats,labels=[1,0,0,0])

ax.text(0, 76, r'\textbf{Arctic}', fontdict=font, bbox=box, horizontalalignment='center',verticalalignment='center')
ax.text(0, 45, r'\textbf{Northern Midlatitudes}', fontdict=font, bbox=box, horizontalalignment='center',verticalalignment='center')
ax.text(0, 0, r'\textbf{Tropics}', fontdict=font, bbox=box, horizontalalignment='center',verticalalignment='center')
ax.text(0, -45, r'\textbf{Southern Midlatitudes}', fontdict=font, bbox=box, horizontalalignment='center',verticalalignment='center')
ax.text(0, -76, r'\textbf{Antarctic}', fontdict=font, bbox=box, horizontalalignment='center',verticalalignment='center')

fig.savefig('figs/earth_map-z.png', dpi=300)

############### Show the plots!

plt.show()
